@@ -1,6 +1,7 @@
 #!/bin/sh
 ##############################################################################
-# Copyright (c) 2016,2018 Free Software Foundation, Inc.                     #
+# Copyright 2018-2020,2022 Thomas E. Dickey                                  #
+# Copyright 2016,2018 Free Software Foundation, Inc.                         #
 #                                                                            #
 # Permission is hereby granted, free of charge, to any person obtaining a    #
 # copy of this software and associated documentation files (the "Software"), #
@@ -27,7 +28,7 @@
 # authorization.                                                             #
 ##############################################################################
 #
-# $Id: library-cfg.sh,v 1.4 2018/11/10 22:58:52 tom Exp $
+# $Id: library-cfg.sh,v 1.8 2022/07/16 17:03:59 tom Exp $
 #
 # Work around incompatible behavior introduced with gnat6, which causes
 # gnatmake to attempt to compile all of the C objects which might be part of
@@ -55,9 +56,18 @@ done
 SHARE="-- "
 test "x$model" = "xdynamic" && SHARE=
 
-sed \
-	-e '/for Library_Options use /s,-- ,'"$SHARE"',' \
-	-e '/for Default_Switches ("C") use/s,-- ,,' \
-	-e '/for Default_Switches ("C") use/s% use .*'%" use($param);"% \
-	$input
-exit 0
+SCRIPT=library-cfg.tmp
+cat >$SCRIPT <<EOF
+/for Library_Options use /{
+	s,-- ,$SHARE,
+}
+/for Default_Switches ("C") use/{
+	s,-- ,,
+	s% use .*% use($param);%
+}
+EOF
+
+sed -f "$SCRIPT" "$input"
+rc=$?
+rm -f "$SCRIPT"
+exit $rc
