@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2020 Thomas E. Dickey                                          *
+ * Copyright 2020,2024 Thomas E. Dickey                                     *
  * Copyright 1998-2016,2017 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -31,7 +31,7 @@
  *   Author:  Juergen Pfeifer, 1995,1997                                    *
  ****************************************************************************/
 
-/* $Id: menu.priv.h,v 1.29 2020/05/24 01:40:20 anonymous.maarten Exp $ */
+/* $Id: menu.priv.h,v 1.30 2024/08/03 15:41:22 tom Exp $ */
 
 /***************************************************************************
 * Module menu.priv.h                                                       *
@@ -61,9 +61,6 @@ extern MENU_EXPORT_VAR(MENU) _nc_Default_Menu;
 /* Normalize menu to default if none was given */
 #define Normalize_Menu( menu ) ((menu)=(menu)?(menu):&_nc_Default_Menu)
 
-#define Get_Menu_Screen( menu ) (menu->userwin ? \
-				 _nc_screen_of(menu->userwin) : CURRENT_SCREEN)
-
 /* Get the user defined (framing) window of the menu */
 #define Get_Menu_UserWin(menu) ((menu)->userwin ? \
     (menu)->userwin : CURRENT_SCREEN->_stdscr)
