@@ -1,5 +1,6 @@
 /****************************************************************************
- * Copyright (c) 1998-2016,2017 Free Software Foundation, Inc.              *
+ * Copyright 2020-2015,2025 Thomas E. Dickey                                *
+ * Copyright 1998-2016,2017 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -42,7 +43,7 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: lib_chgat.c,v 1.12 2017/03/16 23:45:20 tom Exp $")
+MODULE_ID("$Id: lib_chgat.c,v 1.15 2025/02/15 20:53:36 tom Exp $")
 
 NCURSES_EXPORT(int)
 wchgat(WINDOW *win,
@@ -61,7 +62,8 @@ wchgat(WINDOW *win,
        color_pair));
 
     set_extended_pair(opts, color_pair);
-    if (win) {
+    if (win != NULL
+	&& color_pair >= 0) {
 	struct ldat *line = &(win->_line[win->_cury]);
 	int i;
 
