@@ -1,5 +1,6 @@
 /****************************************************************************
- * Copyright (c) 1998-2016,2017 Free Software Foundation, Inc.              *
+ * Copyright 2018-2024,2025 Thomas E. Dickey                                *
+ * Copyright 1998-2016,2017 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -48,7 +49,7 @@
 
 #include <tic.h>
 
-MODULE_ID("$Id: lib_newterm.c,v 1.100 2017/07/22 23:19:00 tom Exp $")
+MODULE_ID("$Id: lib_newterm.c,v 1.108 2025/02/20 01:02:09 tom Exp $")
 
 #ifdef USE_TERM_DRIVER
 #define NumLabels      InfoOf(SP_PARM).numlabels
@@ -87,6 +88,12 @@ _nc_initscr(NCURSES_SP_DCL0)
 	buf.c_oflag &= (unsigned) ~(ONLCR);
 #elif HAVE_SGTTY_H
 	buf.sg_flags &= ~(ECHO | CRMOD);
+#elif defined(EXP_WIN32_DRIVER)
+	buf.dwFlagIn = CONMODE_IN_DEFAULT;
+	buf.dwFlagOut = CONMODE_OUT_DEFAULT | VT_FLAG_OUT;
+	if (WINCONSOLE.isTermInfoConsole) {
+	    buf.dwFlagIn |= VT_FLAG_IN;
+	}
 #else
 	memset(&buf, 0, sizeof(buf));
 #endif
@@ -163,17 +170,17 @@ nofilter(void)
 
 NCURSES_EXPORT(SCREEN *)
 NCURSES_SP_NAME(newterm) (NCURSES_SP_DCLx
-			  NCURSES_CONST char *name,
+			  const char *name,
 			  FILE *ofp,
 			  FILE *ifp)
 {
     int errret;
-    SCREEN *result = 0;
+    SCREEN *result = NULL;
     SCREEN *current;
     TERMINAL *its_term;
     FILE *_ofp = ofp ? ofp : stdout;
     FILE *_ifp = ifp ? ifp : stdin;
-    TERMINAL *new_term = 0;
+    TERMINAL *new_term = NULL;
 
     START_TRACE();
     T((T_CALLED("newterm(%p, \"%s\", %p,%p)"),
@@ -183,8 +190,8 @@ NCURSES_SP_NAME(newterm) (NCURSES_SP_DCLx
        (void *) ifp));
 
 #if NCURSES_SP_FUNCS
-    assert(SP_PARM != 0);
-    if (SP_PARM == 0)
+    assert(SP_PARM != NULL);
+    if (SP_PARM == NULL)
 	returnSP(SP_PARM);
 #endif
 
@@ -192,7 +199,12 @@ NCURSES_SP_NAME(newterm) (NCURSES_SP_DCLx
     _nc_lock_global(curses);
 
     current = CURRENT_SCREEN;
-    its_term = (current ? current->_term : 0);
+    its_term = (current ? current->_term : NULL);
+
+#if defined(EXP_WIN32_DRIVER)
+    _setmode(fileno(_ifp), _O_BINARY);
+    _setmode(fileno(_ofp), _O_BINARY);
+#endif
 
     INIT_TERM_DRIVER();
     /* this loads the capability entry, then sets LINES and COLS */
@@ -200,11 +212,11 @@ NCURSES_SP_NAME(newterm) (NCURSES_SP_DCLx
 	   TINFO_SETUP_TERM(&new_term, name,
 			    fileno(_ofp), &errret, FALSE) != ERR) {
 	int slk_format;
-	int filter_mode;
+	bool filter_mode;
 
-	_nc_set_screen(0);
+	_nc_set_screen(NULL);
 #ifdef USE_TERM_DRIVER
-	assert(new_term != 0);
+	assert(new_term != NULL);
 #endif
 
 #if NCURSES_SP_FUNCS
@@ -229,7 +241,7 @@ NCURSES_SP_NAME(newterm) (NCURSES_SP_DCLx
 						 filter_mode,
 						 slk_format) == ERR) {
 	    _nc_set_screen(current);
-	    result = 0;
+	    result = NULL;
 	} else {
 	    int value;
 	    int cols;
@@ -239,7 +251,7 @@ NCURSES_SP_NAME(newterm) (NCURSES_SP_DCLx
 #elif !NCURSES_SP_FUNCS
 	    _nc_set_screen(CURRENT_SCREEN);
 #endif
-	    assert(SP_PARM != 0);
+	    assert(SP_PARM != NULL);
 	    cols = *(ptrCols(SP_PARM));
 #ifdef USE_TERM_DRIVER
 	    _nc_set_screen(SP_PARM);
@@ -267,6 +279,7 @@ NCURSES_SP_NAME(newterm) (NCURSES_SP_DCLx
 
 	    /* allow user to set maximum escape delay from the environment */
 	    if ((value = _nc_getenv_num("ESCDELAY")) >= 0) {
+		value = Min(value, MAX_DELAY_MSECS);
 #if NCURSES_EXT_FUNCS
 		NCURSES_SP_NAME(set_escdelay) (NCURSES_SP_ARGx value);
 #else
@@ -305,7 +318,7 @@ NCURSES_SP_NAME(newterm) (NCURSES_SP_DCLx
 
 	    NCURSES_SP_NAME(baudrate) (NCURSES_SP_ARG);		/* sets a field in the screen structure */
 
-	    SP_PARM->_keytry = 0;
+	    SP_PARM->_keytry = NULL;
 
 	    /* compute movement costs so we can do better move optimization */
 #ifdef USE_TERM_DRIVER
@@ -320,7 +333,7 @@ NCURSES_SP_NAME(newterm) (NCURSES_SP_DCLx
 	     * properly if we remove rmso or rmul.  Curses applications
 	     * shouldn't be looking at this detail.
 	     */
-#define SGR0_TEST(mode) (mode != 0) && (exit_attribute_mode == 0 || strcmp(mode, exit_attribute_mode))
+#define SGR0_TEST(mode) (mode != NULL) && (exit_attribute_mode == NULL || strcmp(mode, exit_attribute_mode))
 	    SP_PARM->_use_rmso = SGR0_TEST(exit_standout_mode);
 	    SP_PARM->_use_rmul = SGR0_TEST(exit_underline_mode);
 #if USE_ITALIC
@@ -347,14 +360,17 @@ NCURSES_SP_NAME(newterm) (NCURSES_SP_DCLx
 
 #if NCURSES_SP_FUNCS
 NCURSES_EXPORT(SCREEN *)
-newterm(NCURSES_CONST char *name, FILE *ofp, FILE *ifp)
+newterm(const char *name, FILE *ofp, FILE *ifp)
 {
     SCREEN *rc;
+
+    _nc_init_pthreads();
     _nc_lock_global(prescreen);
     START_TRACE();
     rc = NCURSES_SP_NAME(newterm) (CURRENT_SCREEN_PRE, name, ofp, ifp);
     _nc_forget_prescr();
     _nc_unlock_global(prescreen);
+
     return rc;
 }
 #endif
