@@ -1,5 +1,6 @@
 /****************************************************************************
- * Copyright (c) 1998-2015,2016 Free Software Foundation, Inc.              *
+ * Copyright 2020-2024,2025 Thomas E. Dickey                                *
+ * Copyright 1998-2015,2016 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -45,7 +46,7 @@
 #define CUR SP_TERMTYPE
 #endif
 
-MODULE_ID("$Id: resizeterm.c,v 1.49 2016/05/28 23:11:26 tom Exp $")
+MODULE_ID("$Id: resizeterm.c,v 1.60 2025/11/12 01:27:41 tom Exp $")
 
 /*
  * If we're trying to be reentrant, do not want any local statics.
@@ -102,8 +103,9 @@ NCURSES_EXPORT(bool)
 NCURSES_SP_NAME(is_term_resized) (NCURSES_SP_DCLx int ToLines, int ToCols)
 {
     T((T_CALLED("is_term_resized(%p, %d, %d)"), (void *) SP_PARM, ToLines, ToCols));
-    returnCode(ToLines > 0
+    returnBool(ToLines > 0
 	       && ToCols > 0
+	       && SP_PARM != NULL
 	       && (ToLines != screen_lines(SP_PARM)
 		   || ToCols != screen_columns(SP_PARM)));
 }
@@ -121,10 +123,10 @@ is_term_resized(int ToLines, int ToCols)
 static ripoff_t *
 ripped_window(WINDOW *win)
 {
-    ripoff_t *result = 0;
+    ripoff_t *result = NULL;
     ripoff_t *rop;
 
-    if (win != 0) {
+    if (win != NULL) {
 #ifdef USE_SP_RIPOFF
 	SCREEN *sp = _nc_screen_of(win);
 #endif
@@ -147,7 +149,7 @@ ripped_bottom(WINDOW *win)
 {
     int result = 0;
 
-    if (win != 0) {
+    if (win != NULL) {
 	ripoff_t *rop;
 
 #ifdef USE_SP_RIPOFF
@@ -173,7 +175,7 @@ child_depth(WINDOW *cmp)
 {
     int depth = 0;
 
-    if (cmp != 0) {
+    if (cmp != NULL) {
 #ifdef USE_SP_WINDOWLIST
 	SCREEN *sp = _nc_screen_of(cmp);
 #endif
@@ -198,9 +200,9 @@ parent_depth(WINDOW *cmp)
 {
     int depth = 0;
 
-    if (cmp != 0) {
+    if (cmp != NULL) {
 	WINDOW *tst;
-	while ((tst = cmp->_parent) != 0) {
+	while ((tst = cmp->_parent) != NULL) {
 	    ++depth;
 	    cmp = tst;
 	}
@@ -209,7 +211,7 @@ parent_depth(WINDOW *cmp)
 }
 
 /*
- * FIXME: must adjust position so it's within the parent!
+ * FIXME: must adjust position so it is within the parent!
  */
 static int
 adjust_window(WINDOW *win, int ToLines, int ToCols, int stolen EXTRA_DCLS)
@@ -218,17 +220,17 @@ adjust_window(WINDOW *win, int ToLines, int ToCols, int stolen EXTRA_DCLS)
     int bottom = CurLines + _nc_screen_of(win)->_topstolen - stolen;
     int myLines = win->_maxy + 1;
     int myCols = win->_maxx + 1;
-    ripoff_t *rop = ripped_window(win);
+    const ripoff_t *rop = ripped_window(win);
 
     T((T_CALLED("adjust_window(%p,%d,%d)%s depth %d/%d currently %ldx%ld at %ld,%ld"),
        (void *) win, ToLines, ToCols,
-       (rop != 0) ? " (rip)" : "",
+       (rop != NULL) ? " (rip)" : "",
        parent_depth(win),
        child_depth(win),
        (long) getmaxy(win), (long) getmaxx(win),
        (long) getbegy(win) + win->_yoffset, (long) getbegx(win)));
 
-    if (rop != 0 && rop->line < 0) {
+    if (rop != NULL && rop->line < 0) {
 	/*
 	 * If it is a ripped-off window at the bottom of the screen, simply
 	 * move it to the same relative position.
@@ -292,7 +294,7 @@ decrease_size(NCURSES_SP_DCLx int ToLines, int ToCols, int stolen EXTRA_DCLS)
 	for (each_window(SP_PARM, wp)) {
 	    WINDOW *win = &(wp->win);
 
-	    if (!(win->_flags & _ISPAD)) {
+	    if (!IS_PAD(win)) {
 		if (child_depth(win) == depth) {
 		    found = TRUE;
 		    if (adjust_window(win, ToLines, ToCols,
@@ -327,7 +329,7 @@ increase_size(NCURSES_SP_DCLx int ToLines, int ToCols, int stolen EXTRA_DCLS)
 	for (each_window(SP_PARM, wp)) {
 	    WINDOW *win = &(wp->win);
 
-	    if (!(win->_flags & _ISPAD)) {
+	    if (!IS_PAD(win)) {
 		if (parent_depth(win) == depth) {
 		    found = TRUE;
 		    if (adjust_window(win, ToLines, ToCols,
@@ -342,7 +344,7 @@ increase_size(NCURSES_SP_DCLx int ToLines, int ToCols, int stolen EXTRA_DCLS)
 }
 
 /*
- * This function reallocates NCURSES window structures, with no side-effects
+ * This function reallocates NCURSES window structures, with no side effects
  * such as ungetch().
  */
 NCURSES_EXPORT(int)
@@ -353,10 +355,10 @@ NCURSES_SP_NAME(resize_term) (NCURSES_SP_DCLx int ToLines, int ToCols)
 
     T((T_CALLED("resize_term(%p,%d,%d) old(%d,%d)"),
        (void *) SP_PARM, ToLines, ToCols,
-       (SP_PARM == 0) ? -1 : screen_lines(SP_PARM),
-       (SP_PARM == 0) ? -1 : screen_columns(SP_PARM)));
+       (SP_PARM == NULL) ? -1 : screen_lines(SP_PARM),
+       (SP_PARM == NULL) ? -1 : screen_columns(SP_PARM)));
 
-    if (SP_PARM == 0 || ToLines <= 0 || ToCols <= 0) {
+    if (SP_PARM == NULL || ToLines <= 0 || ToCols <= 0) {
 	returnCode(ERR);
     }
 
@@ -408,8 +410,8 @@ NCURSES_SP_NAME(resize_term) (NCURSES_SP_DCLx int ToLines, int ToCols)
 #ifdef USE_TERM_DRIVER
 	    CallDriver_2(SP_PARM, td_setsize, ToLines, ToCols);
 #else
-	    lines = (NCURSES_SIZE_T) ToLines;
-	    columns = (NCURSES_SIZE_T) ToCols;
+	    lines = (NCURSES_INT2) ToLines;
+	    columns = (NCURSES_INT2) ToCols;
 #endif
 
 	    SP_PARM->_lines_avail = (NCURSES_SIZE_T) (ToLines - was_stolen);
@@ -472,18 +474,18 @@ NCURSES_SP_NAME(resizeterm) (NCURSES_SP_DCLx int ToLines, int ToCols)
 
     T((T_CALLED("resizeterm(%p, %d,%d) old(%d,%d)"),
        (void *) SP_PARM, ToLines, ToCols,
-       (SP_PARM == 0) ? -1 : screen_lines(SP_PARM),
-       (SP_PARM == 0) ? -1 : screen_columns(SP_PARM)));
+       (SP_PARM == NULL) ? -1 : screen_lines(SP_PARM),
+       (SP_PARM == NULL) ? -1 : screen_columns(SP_PARM)));
 
-    if (SP_PARM != 0 && ToLines > 0 && ToCols > 0) {
+    if (SP_PARM != NULL && ToLines > 0 && ToCols > 0) {
 	result = OK;
 	SP_PARM->_sig_winch = FALSE;
 
 	if (NCURSES_SP_NAME(is_term_resized) (NCURSES_SP_ARGx ToLines, ToCols)) {
 #if USE_SIGWINCH
 	    ripoff_t *rop;
-	    bool slk_visible = (SP_PARM != 0
-				&& SP_PARM->_slk != 0
+	    bool slk_visible = (SP_PARM != NULL
+				&& SP_PARM->_slk != NULL
 				&& !(SP_PARM->_slk->hidden));
 
 	    if (slk_visible) {
@@ -504,7 +506,7 @@ NCURSES_SP_NAME(resizeterm) (NCURSES_SP_DCLx int ToLines, int ToCols)
 	     */
 	    for (each_ripoff(rop)) {
 		if (rop->win != StdScreen(SP_PARM)
-		    && rop->win != 0
+		    && rop->win != NULL
 		    && rop->line < 0) {
 
 		    if (rop->hook != _nc_slk_initialize) {
