@@ -1,5 +1,6 @@
 /****************************************************************************
- * Copyright (c) 1998-2016,2017 Free Software Foundation, Inc.              *
+ * Copyright 2020-2024,2025 Thomas E. Dickey                                *
+ * Copyright 1998-2016,2017 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -83,7 +84,7 @@
 #undef USE_OLD_TTY
 #endif /* USE_OLD_TTY */
 
-MODULE_ID("$Id: lib_baudrate.c,v 1.43 2017/03/31 17:19:30 tom Exp $")
+MODULE_ID("$Id: lib_baudrate.c,v 1.48 2025/10/18 17:56:35 tom Exp $")
 
 /*
  *	int
@@ -98,6 +99,7 @@ struct speed {
     int actual_speed;		/* the actual speed */
 };
 
+#if !defined(_NC_WINDOWS)
 #define DATA(number) { B##number, number }
 
 static struct speed const speeds[] =
@@ -187,10 +189,16 @@ static struct speed const speeds[] =
 #endif
 #endif
 };
+#endif /* !EXP_WIN32_DRIVER */
 
 NCURSES_EXPORT(int)
 _nc_baudrate(int OSpeed)
 {
+#if defined(_NC_WINDOWS)
+    /* On Windows this is a noop */
+    (void) OSpeed;
+    return (OK);
+#else
 #if !USE_REENTRANT
     static int last_OSpeed;
     static int last_baudrate;
@@ -229,13 +237,16 @@ _nc_baudrate(int OSpeed)
 #endif
     }
     return (result);
+#endif /* !EXP_WIN32_DRIVER */
 }
 
 NCURSES_EXPORT(int)
 _nc_ospeed(int BaudRate)
 {
     int result = 1;
-
+#if defined(_NC_WINDOWS)
+    (void) BaudRate;
+#else
     if (BaudRate >= 0) {
 	unsigned i;
 
@@ -246,6 +257,7 @@ _nc_ospeed(int BaudRate)
 	    }
 	}
     }
+#endif
     return (result);
 }
 
@@ -256,6 +268,10 @@ NCURSES_SP_NAME(baudrate) (NCURSES_SP_DCL0)
 
     T((T_CALLED("baudrate(%p)"), (void *) SP_PARM));
 
+#if defined(_NC_WINDOWS)
+    (void) SP_PARM;
+    result = OK;
+#else
     /*
      * In debugging, allow the environment symbol to override when we're
      * redirecting to a file, so we can construct repeatable test-cases
@@ -264,7 +280,7 @@ NCURSES_SP_NAME(baudrate) (NCURSES_SP_DCL0)
 #ifdef TRACE
     if (IsValidTIScreen(SP_PARM)
 	&& !NC_ISATTY(fileno((SP_PARM && SP_PARM->_ofp) ? SP_PARM->_ofp : stdout))
-	&& getenv("BAUDRATE") != 0) {
+	&& getenv("BAUDRATE") != NULL) {
 	int ret;
 	if ((ret = _nc_getenv_num("BAUDRATE")) <= 0)
 	    ret = 9600;
@@ -289,7 +305,7 @@ NCURSES_SP_NAME(baudrate) (NCURSES_SP_DCL0)
     } else {
 	result = ERR;
     }
-
+#endif /* !EXP_WIN32_DRIVER */
     returnCode(result);
 }
 
