@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2020 Thomas E. Dickey                                          *
+ * Copyright 2020-2024,2025 Thomas E. Dickey                                *
  * Copyright 1998-2016,2017 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -50,14 +50,13 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: lib_raw.c,v 1.25 2020/09/05 21:29:10 tom Exp $")
+MODULE_ID("$Id: lib_raw.c,v 1.34 2025/10/18 17:30:15 tom Exp $")
 
 #if HAVE_SYS_TERMIO_H
 #include <sys/termio.h>		/* needed for ISC */
 #endif
 
 #ifdef __EMX__
-#include <io.h>
 #define _nc_setmode(mode) setmode(SP_PARM->_ifd, mode)
 #else
 #define _nc_setmode(mode)	/* nothing */
@@ -85,7 +84,7 @@ NCURSES_SP_NAME(raw) (NCURSES_SP_DCL0)
     TERMINAL *termp;
 
     T((T_CALLED("raw(%p)"), (void *) SP_PARM));
-    if ((termp = TerminalOf(SP_PARM)) != 0) {
+    if ((termp = TerminalOf(SP_PARM)) != NULL) {
 	TTY buf;
 
 	BEFORE("raw");
@@ -97,8 +96,8 @@ NCURSES_SP_NAME(raw) (NCURSES_SP_DCL0)
 	buf.c_iflag &= (unsigned) ~(COOKED_INPUT);
 	buf.c_cc[VMIN] = 1;
 	buf.c_cc[VTIME] = 0;
-#elif defined(EXP_WIN32_DRIVER)
-	buf.dwFlagIn &= ~CONMODE_NORAW;
+#elif defined(_NC_WINDOWS)
+	buf.dwFlagIn &= (unsigned long) ~CONMODE_NORAW;
 #else
 	buf.sg_flags |= RAW;
 #endif
@@ -116,8 +115,8 @@ NCURSES_SP_NAME(raw) (NCURSES_SP_DCL0)
 	    KbdSetStatus(&kbdinfo, 0);
 #endif
 	    if (SP_PARM) {
-		SP_PARM->_raw = TRUE;
-		SP_PARM->_cbreak = 1;
+		IsRaw(SP_PARM) = TRUE;
+		IsCbreak(SP_PARM) = 1;
 	    }
 	    termp->Nttyb = buf;
 	}
@@ -141,7 +140,7 @@ NCURSES_SP_NAME(cbreak) (NCURSES_SP_DCL0)
     TERMINAL *termp;
 
     T((T_CALLED("cbreak(%p)"), (void *) SP_PARM));
-    if ((termp = TerminalOf(SP_PARM)) != 0) {
+    if ((termp = TerminalOf(SP_PARM)) != NULL) {
 	TTY buf;
 
 	BEFORE("cbreak");
@@ -151,19 +150,18 @@ NCURSES_SP_NAME(cbreak) (NCURSES_SP_DCL0)
 #ifdef TERMIOS
 	buf.c_lflag &= (unsigned) ~ICANON;
 	buf.c_iflag &= (unsigned) ~ICRNL;
-	buf.c_lflag |= ISIG;
 	buf.c_cc[VMIN] = 1;
 	buf.c_cc[VTIME] = 0;
-#elif defined(EXP_WIN32_DRIVER)
+#elif defined(_NC_WINDOWS)
 	buf.dwFlagIn |= CONMODE_NORAW;
-	buf.dwFlagIn &= ~CONMODE_NOCBREAK;
+	buf.dwFlagIn &= (unsigned long) ~CONMODE_NOCBREAK;
 #else
 	buf.sg_flags |= CBREAK;
 #endif
 	result = NCURSES_SP_NAME(_nc_set_tty_mode) (NCURSES_SP_ARGx &buf);
 	if (result == OK) {
 	    if (SP_PARM) {
-		SP_PARM->_cbreak = 1;
+		IsCbreak(SP_PARM) = 1;
 	    }
 	    termp->Nttyb = buf;
 	}
@@ -180,17 +178,13 @@ cbreak(void)
 }
 #endif
 
-/*
- * Note:
- * this implementation may be wrong.  See the comment under intrflush().
- */
 NCURSES_EXPORT(void)
 NCURSES_SP_NAME(qiflush) (NCURSES_SP_DCL0)
 {
     TERMINAL *termp;
 
     T((T_CALLED("qiflush(%p)"), (void *) SP_PARM));
-    if ((termp = TerminalOf(SP_PARM)) != 0) {
+    if ((termp = TerminalOf(SP_PARM)) != NULL) {
 	TTY buf;
 	int result;
 
@@ -225,7 +219,7 @@ NCURSES_SP_NAME(noraw) (NCURSES_SP_DCL0)
     TERMINAL *termp;
 
     T((T_CALLED("noraw(%p)"), (void *) SP_PARM));
-    if ((termp = TerminalOf(SP_PARM)) != 0) {
+    if ((termp = TerminalOf(SP_PARM)) != NULL) {
 	TTY buf;
 
 	BEFORE("noraw");
@@ -236,7 +230,7 @@ NCURSES_SP_NAME(noraw) (NCURSES_SP_DCL0)
 	buf.c_lflag |= ISIG | ICANON |
 	    (termp->Ottyb.c_lflag & IEXTEN);
 	buf.c_iflag |= COOKED_INPUT;
-#elif defined(EXP_WIN32_DRIVER)
+#elif defined(_NC_WINDOWS)
 	buf.dwFlagIn |= CONMODE_NORAW;
 #else
 	buf.sg_flags &= ~(RAW | CBREAK);
@@ -255,8 +249,8 @@ NCURSES_SP_NAME(noraw) (NCURSES_SP_DCL0)
 	    KbdSetStatus(&kbdinfo, 0);
 #endif
 	    if (SP_PARM) {
-		SP_PARM->_raw = FALSE;
-		SP_PARM->_cbreak = 0;
+		IsRaw(SP_PARM) = FALSE;
+		IsCbreak(SP_PARM) = 0;
 	    }
 	    termp->Nttyb = buf;
 	}
@@ -280,7 +274,7 @@ NCURSES_SP_NAME(nocbreak) (NCURSES_SP_DCL0)
     TERMINAL *termp;
 
     T((T_CALLED("nocbreak(%p)"), (void *) SP_PARM));
-    if ((termp = TerminalOf(SP_PARM)) != 0) {
+    if ((termp = TerminalOf(SP_PARM)) != NULL) {
 	TTY buf;
 
 	BEFORE("nocbreak");
@@ -290,7 +284,7 @@ NCURSES_SP_NAME(nocbreak) (NCURSES_SP_DCL0)
 #ifdef TERMIOS
 	buf.c_lflag |= ICANON;
 	buf.c_iflag |= ICRNL;
-#elif defined(EXP_WIN32_DRIVER)
+#elif defined(_NC_WINDOWS)
 	buf.dwFlagIn |= (CONMODE_NOCBREAK | CONMODE_NORAW);
 #else
 	buf.sg_flags &= ~CBREAK;
@@ -298,7 +292,7 @@ NCURSES_SP_NAME(nocbreak) (NCURSES_SP_DCL0)
 	result = NCURSES_SP_NAME(_nc_set_tty_mode) (NCURSES_SP_ARGx &buf);
 	if (result == OK) {
 	    if (SP_PARM) {
-		SP_PARM->_cbreak = 0;
+		IsCbreak(SP_PARM) = 0;
 	    }
 	    termp->Nttyb = buf;
 	}
@@ -321,7 +315,7 @@ NCURSES_SP_NAME(noqiflush) (NCURSES_SP_DCL0)
     TERMINAL *termp;
 
     T((T_CALLED("noqiflush(%p)"), (void *) SP_PARM));
-    if ((termp = TerminalOf(SP_PARM)) != 0) {
+    if ((termp = TerminalOf(SP_PARM)) != NULL) {
 	TTY buf;
 	int result;
 
@@ -350,11 +344,7 @@ noqiflush(void)
 #endif
 
 /*
- * This call does the same thing as the qiflush()/noqiflush() pair.  We know
- * for certain that SVr3 intrflush() tweaks the NOFLSH bit; on the other hand,
- * the match (in the SVr4 man pages) between the language describing NOFLSH in
- * termio(7) and the language describing qiflush()/noqiflush() in
- * curs_inopts(3x) is too exact to be coincidence.
+ * This call does the same thing as the qiflush()/noqiflush() pair.
  */
 NCURSES_EXPORT(int)
 NCURSES_SP_NAME(intrflush) (NCURSES_SP_DCLx WINDOW *win GCC_UNUSED, bool flag)
@@ -363,10 +353,10 @@ NCURSES_SP_NAME(intrflush) (NCURSES_SP_DCLx WINDOW *win GCC_UNUSED, bool flag)
     TERMINAL *termp;
 
     T((T_CALLED("intrflush(%p,%d)"), (void *) SP_PARM, flag));
-    if (SP_PARM == 0)
+    if (SP_PARM == NULL)
 	returnCode(ERR);
 
-    if ((termp = TerminalOf(SP_PARM)) != 0) {
+    if ((termp = TerminalOf(SP_PARM)) != NULL) {
 	TTY buf;
 
 	BEFORE("intrflush");
@@ -395,3 +385,46 @@ intrflush(WINDOW *win GCC_UNUSED, bool flag)
     return NCURSES_SP_NAME(intrflush) (CURRENT_SCREEN, win, flag);
 }
 #endif
+
+#if NCURSES_EXT_FUNCS
+/* *INDENT-OFF* */
+
+/*
+ * SCREEN is always opaque, but nl/raw/cbreak/echo set properties in it.
+ * As an extension, provide a way to query the properties.
+ *
+ * There are other properties which could be queried, e.g., filter, keypad,
+ * use_env, use_meta, but these particular properties are saved/restored within
+ * the wgetnstr() and wgetn_wstr() functions, which requires that the higher
+ * level curses library knows about the internal state of the lower level
+ * terminfo library.
+ */
+
+#define is_TEST(show,what) \
+    NCURSES_EXPORT(int) \
+    NCURSES_SP_NAME(show) (NCURSES_SP_DCL0) \
+    { \
+	return ((SP_PARM != NULL) ? (what(SP_PARM) ? 1 : 0) : -1); \
+    }
+
+is_TEST(is_nl, IsNl)
+is_TEST(is_raw, IsRaw)
+is_TEST(is_cbreak, IsCbreak)
+is_TEST(is_echo, IsEcho)
+
+#if NCURSES_SP_FUNCS
+#undef is_TEST
+#define is_TEST(show) \
+    NCURSES_EXPORT(int) \
+    show(void) \
+    { \
+	return NCURSES_SP_NAME(show) (CURRENT_SCREEN); \
+    }
+is_TEST(is_nl)
+is_TEST(is_raw)
+is_TEST(is_cbreak)
+is_TEST(is_echo)
+#endif
+
+/* *INDENT-ON* */
+#endif /* extensions */
