@@ -1,5 +1,6 @@
 /****************************************************************************
- * Copyright (c) 1998-2003,2009 Free Software Foundation, Inc.              *
+ * Copyright 2020-2024,2025 Thomas E. Dickey                                *
+ * Copyright 1998-2003,2009 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -28,21 +29,21 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: lib_termname.c,v 1.12 2009/10/24 21:56:58 tom Exp $")
+MODULE_ID("$Id: lib_termname.c,v 1.15 2025/01/12 00:41:56 tom Exp $")
 
 NCURSES_EXPORT(char *)
 NCURSES_SP_NAME(termname) (NCURSES_SP_DCL0)
 {
-    char *name = 0;
+    char *name = NULL;
 
     T((T_CALLED("termname(%p)"), (void *) SP_PARM));
 
 #if NCURSES_SP_FUNCS
-    if (TerminalOf(SP_PARM) != 0) {
+    if (TerminalOf(SP_PARM) != NULL) {
 	name = TerminalOf(SP_PARM)->_termname;
     }
 #else
-    if (cur_term != 0)
+    if (cur_term != NULL)
 	name = cur_term->_termname;
 #endif
 
