@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2019,2020 Thomas E. Dickey                                     *
+ * Copyright 2019-2024,2025 Thomas E. Dickey                                *
  * Copyright 1998-2012,2015 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -35,16 +35,12 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: lib_tracebits.c,v 1.30 2020/09/05 22:54:47 tom Exp $")
+MODULE_ID("$Id: lib_tracebits.c,v 1.36 2025/10/18 18:10:04 tom Exp $")
 
 #if HAVE_SYS_TERMIO_H
 #include <sys/termio.h>		/* needed for ISC */
 #endif
 
-#ifdef __EMX__
-#include <io.h>
-#endif
-
 /* may be undefined if we're using termio.h */
 #ifndef TOSTOP
 #define TOSTOP 0
@@ -72,21 +68,21 @@ MODULE_ID("$Id: lib_tracebits.c,v 1.30 2020/09/05 22:54:47 tom Exp $")
 
 #ifdef TRACE
 
-#if defined(EXP_WIN32_DRIVER)
+#if defined(_NC_WINDOWS)
 #define BITNAMELEN 36
 #else
 #define BITNAMELEN 8
 #endif
 
 typedef struct {
-    unsigned int val;
+    unsigned long val;
     const char name[BITNAMELEN];
 } BITNAMES;
 
 #define TRACE_BUF_SIZE(num) (_nc_globals.tracebuf_ptr[num].size)
 
 static void
-lookup_bits(char *buf, const BITNAMES * table, const char *label, unsigned int val)
+lookup_bits(char *buf, const BITNAMES * table, const char *label, unsigned long val)
 {
     const BITNAMES *sp;
 
@@ -104,7 +100,7 @@ lookup_bits(char *buf, const BITNAMES * table, const char *label, unsigned int v
 }
 
 NCURSES_EXPORT(char *)
-_nc_trace_ttymode(TTY * tty)
+_nc_trace_ttymode(const TTY * tty)
 /* describe the state of the terminal control bits exactly */
 {
     char *buf;
@@ -172,7 +168,7 @@ _nc_trace_ttymode(TTY * tty)
 			8 + sizeof(cflags) +
 			8 + sizeof(lflags) +
 			8);
-    if (buf != 0) {
+    if (buf != NULL) {
 
 	if (tty->c_iflag & ALLIN)
 	    lookup_bits(buf, iflags, "iflags", tty->c_iflag);
@@ -218,7 +214,7 @@ _nc_trace_ttymode(TTY * tty)
 	if (tty->c_lflag & ALLLOCAL)
 	    lookup_bits(buf, lflags, "lflags", tty->c_lflag);
     }
-#elif defined(EXP_WIN32_DRIVER)
+#elif defined(_NC_WINDOWS)
 #define DATA(name)        { name, { #name } }
     static const BITNAMES dwFlagsOut[] =
     {
@@ -244,7 +240,7 @@ _nc_trace_ttymode(TTY * tty)
     buf = _nc_trace_buf(0,
 			8 + sizeof(dwFlagsOut) +
 			8 + sizeof(dwFlagsIn));
-    if (buf != 0) {
+    if (buf != NULL) {
 	lookup_bits(buf, dwFlagsIn, "dwIn", tty->dwFlagIn);
 	lookup_bits(buf, dwFlagsOut, "dwOut", tty->dwFlagOut);
     }
@@ -284,7 +280,7 @@ _nc_trace_ttymode(TTY * tty)
 
     buf = _nc_trace_buf(0,
 			8 + sizeof(cflags));
-    if (buf != 0) {
+    if (buf != NULL) {
 	if (tty->sg_flags & ALLCTRL) {
 	    lookup_bits(buf, cflags, "cflags", tty->sg_flags);
 	}
