@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2020-2021,2022 Thomas E. Dickey                                *
+ * Copyright 2020-2024,2025 Thomas E. Dickey                                *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -26,7 +26,7 @@
  * authorization.                                                           *
  ****************************************************************************/
 /*
- * $Id: move_field.c,v 1.14 2022/12/10 23:31:31 tom Exp $
+ * $Id: move_field.c,v 1.21 2025/01/11 14:54:49 tom Exp $
  *
  * Demonstrate move_field().
  */
@@ -62,7 +62,7 @@ static struct {
     { CTRL('W'),     REQ_NEXT_WORD,   "go to next word" },
     { CTRL('X'),     REQ_CLR_FIELD,   "clear field" },
     { CTRL('['),     MY_QUIT,         "exit form" },
-    { KEY_F(1),      MY_HELP,         "show this screen", },
+    { HELP_KEY_2,    MY_HELP,         "show this screen", },
     { KEY_BACKSPACE, REQ_DEL_PREV,    "delete previous character" },
     { KEY_BTAB,      REQ_PREV_FIELD,  "go to previous field" },
     { KEY_DOWN,      REQ_DOWN_CHAR,   "move down 1 character" },
@@ -91,19 +91,21 @@ my_help_edit_field(void)
 	const char *code = keyname(commands[n].code);
 	size_t need = 5;
 #ifdef NCURSES_VERSION
-	if ((name = form_request_name(commands[n].result)) == 0)
+	if ((name = form_request_name(commands[n].result)) == NULL)
 #endif
 	    name = commands[n].help;
 	need = 5 + strlen(code) + strlen(name);
 	msg = typeMalloc(char, need);
-	_nc_SPRINTF(msg, _nc_SLIMIT(need) "%s -- %s", code, name);
-	msgs[used++] = msg;
+	if (msg != NULL) {
+	    _nc_SPRINTF(msg, _nc_SLIMIT(need) "%s -- %s", code, name);
+	    msgs[used++] = msg;
+	}
     }
     msgs[used++] =
 	strdup("Arrow keys move within a field as you would expect.");
-    msgs[used] = 0;
+    msgs[used] = NULL;
     popup_msg2(stdscr, msgs);
-    for (n = 0; msgs[n] != 0; ++n) {
+    for (n = 0; msgs[n] != NULL; ++n) {
 	free(msgs[n]);
     }
     free(msgs);
@@ -116,7 +118,7 @@ make_label(const char *label, int frow, int fcol)
 
     if (f) {
 	set_field_buffer(f, 0, label);
-	set_field_opts(f, (int) ((unsigned) field_opts(f) & ~O_ACTIVE));
+	set_field_opts(f, (int) ((unsigned) field_opts(f) & (unsigned) ~O_ACTIVE));
     }
     return (f);
 }
@@ -146,27 +148,27 @@ erase_form(FORM *f)
 }
 
 static FieldAttrs *
-my_field_attrs(FIELD *f)
+my_field_attrs(NCURSES_CONST FIELD *f)
 {
     return (FieldAttrs *) field_userptr(f);
 }
 
 static int
-buffer_length(FIELD *f)
+buffer_length(NCURSES_CONST FIELD *f)
 {
     return my_field_attrs(f)->row_lengths[0];
 }
 
 static void
-set_buffer_length(FIELD *f, int length)
+set_buffer_length(NCURSES_CONST FIELD *f, int length)
 {
     my_field_attrs(f)->row_lengths[0] = length;
 }
 
 static int
-offset_in_field(FORM *form)
+offset_in_field(NCURSES_CONST FORM *form)
 {
-    FIELD *field = current_field(form);
+    NCURSES_CONST FIELD *field = current_field(form);
     int currow, curcol;
 
     form_getyx(form, currow, curcol);
@@ -245,7 +247,7 @@ my_edit_field(FORM *form, int *result)
 
 	default:
 	    modified = (ch < MIN_FORM_COMMAND
-			&& isprint(ch));
+			&& isprint(UChar(ch)));
 	    break;
 	}
 
@@ -275,9 +277,9 @@ copy_fields(FIELD **source, size_t length)
 
 /* display a status message to show what's happening */
 static void
-show_status(FORM *form, FIELD *field)
+show_status(NCURSES_CONST FORM *form, NCURSES_CONST FIELD *field)
 {
-    WINDOW *sub = form_sub(form);
+    NCURSES_CONST WINDOW *sub = form_sub(form);
     int currow, curcol;
 
     getyx(stdscr, currow, curcol);
@@ -444,7 +446,7 @@ demo_forms(void)
 
     all_fields[n] = (FIELD *) 0;
 
-    if ((form = new_form(all_fields)) != 0) {
+    if ((form = new_form(all_fields)) != NULL) {
 	int finished = 0;
 
 	post_form(form);
@@ -466,7 +468,7 @@ demo_forms(void)
 
 	free_form(form);
     }
-    for (c = 0; all_fields[c] != 0; c++) {
+    for (c = 0; all_fields[c] != NULL; c++) {
 	free_edit_field(all_fields[c]);
 	free_field(all_fields[c]);
     }
