@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2019,2020 Thomas E. Dickey                                     *
+ * Copyright 2019-2022,2024 Thomas E. Dickey                                *
  * Copyright 2017 Free Software Foundation, Inc.                            *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -29,7 +29,7 @@
 /*
  * clone of view.c, using pads
  *
- * $Id: padview.c,v 1.16 2020/02/02 23:34:34 tom Exp $
+ * $Id: padview.c,v 1.28 2024/12/21 17:34:13 tom Exp $
  */
 
 #include <test.priv.h>
@@ -41,10 +41,11 @@
 
 #if HAVE_NEWPAD
 
-static void finish(int sig) GCC_NORETURN;
+static GCC_NORETURN void finish(int sig);
 
 #define my_pair 1
 
+static WINDOW *global_pad;
 static int shift = 0;
 static bool try_color = FALSE;
 
@@ -55,8 +56,6 @@ static int num_lines;
 static bool n_option = FALSE;
 #endif
 
-static void usage(void) GCC_NORETURN;
-
 static void
 failed(const char *msg)
 {
@@ -69,6 +68,8 @@ static void
 finish(int sig)
 {
     endwin();
+    if (global_pad != NULL)
+	delwin(global_pad);
     ExitProgram(sig != 0 ? EXIT_FAILURE : EXIT_SUCCESS);
 }
 
@@ -87,8 +88,9 @@ show_all(const char *tag, WINDOW *my_pad, int my_row)
     wattrset(stdscr, COLOR_PAIR(my_pair));
     clear();
 
+    i = (int) (sizeof(temp) - strlen(tag) - 8);
     _nc_SPRINTF(temp, _nc_SLIMIT(sizeof(temp))
-		"view %.*s", (int) strlen(tag), tag);
+		"view %.*s", i, tag);
     i = (int) strlen(temp);
     _nc_SPRINTF(temp + i, _nc_SLIMIT(sizeof(temp) - (size_t) i)
 		" %.*s", (int) sizeof(temp) - i - 2, fname);
@@ -123,7 +125,7 @@ read_file(const char *filename)
     size_t len;
     struct stat sb;
     char *my_blob;
-    char **my_vec = 0;
+    char **my_vec = NULL;
     WINDOW *my_pad;
 
     if (stat(filename, &sb) != 0
@@ -135,18 +137,21 @@ read_file(const char *filename)
 	failed("input is empty");
     }
 
-    if ((fp = fopen(filename, "r")) == 0) {
+    if ((fp = fopen(filename, "r")) == NULL) {
 	failed("cannot open input-file");
     }
 
-    if ((my_blob = malloc((size_t) sb.st_size + 1)) == 0) {
+    if ((my_blob = malloc((size_t) sb.st_size + 1)) == NULL) {
 	failed("cannot allocate memory for input-file");
     }
 
     len = fread(my_blob, sizeof(char), (size_t) sb.st_size, fp);
-    my_blob[sb.st_size] = '\0';
     fclose(fp);
 
+    if (len > (size_t) sb.st_size)
+	len = (size_t) sb.st_size;
+    my_blob[len] = '\0';
+
     for (pass = 0; pass < 2; ++pass) {
 	char *base = my_blob;
 	k = 0;
@@ -160,18 +165,25 @@ read_file(const char *filename)
 		++k;
 	    }
 	}
+	if (base != (my_blob + j)) {
+	    if (pass)
+		my_vec[k] = base;
+	    ++k;
+	}
 	num_lines = k;
-	if (base != (my_blob + j))
-	    ++num_lines;
-	if (!pass &&
-	    ((my_vec = typeCalloc(char *, (size_t) k + 2)) == 0)) {
-	    failed("cannot allocate line-vector #1");
+	if (pass == 0) {
+	    if (((my_vec = typeCalloc(char *, (size_t) k + 2)) == NULL)) {
+		failed("cannot allocate line-vector #1");
+	    }
+	} else {
+	    if (my_vec[0] == NULL)
+		my_vec[0] = my_blob;
 	}
     }
 
 #if USE_WIDEC_SUPPORT
     if (!memcmp("\357\273\277", my_blob, 3)) {
-	char *s = my_blob + 3;
+	const char *s = my_blob + 3;
 	char *d = my_blob;
 	Trace(("trim BOM"));
 	do {
@@ -188,7 +200,7 @@ read_file(const char *filename)
     }
     width = (width + 1) * 5;
     my_pad = newpad(height, width);
-    if (my_pad == 0)
+    if (my_pad == NULL)
 	failed("cannot allocate pad workspace");
     if (try_color) {
 	wattrset(my_pad, COLOR_PAIR(my_pair));
@@ -202,7 +214,7 @@ read_file(const char *filename)
     for (k = 0; my_vec[k]; ++k) {
 	char *s;
 #if USE_WIDEC_SUPPORT
-	char *last = my_vec[k] + (int) strlen(my_vec[k]);
+	const char *last = my_vec[k] + (int) strlen(my_vec[k]);
 	wchar_t wch[2];
 	size_t rc;
 #ifndef state_unused
@@ -237,34 +249,38 @@ read_file(const char *filename)
 }
 
 static void
-usage(void)
+usage(int ok)
 {
     static const char *msg[] =
     {
 	"Usage: view [options] file"
 	,""
+	,USAGE_COMMON
 	,"Options:"
-	," -c       use color if terminal supports it"
+	," -C       use color if terminal supports it"
 	," -i       ignore INT, QUIT, TERM signals"
 #if USE_WIDEC_SUPPORT
 	," -n       use waddch (bytes) rather then wadd_wch (wide-chars)"
 #endif
-	," -s       start in single-step mode, waiting for input"
+	," -s       start in single-step mode"
 #ifdef TRACE
 	," -t       trace screen updates"
-	," -T NUM   specify trace mask"
+	," -D NUM   specify trace mask"
 #endif
     };
     size_t n;
     for (n = 0; n < SIZEOF(msg); n++)
 	fprintf(stderr, "%s\n", msg[n]);
-    ExitProgram(EXIT_FAILURE);
+    ExitProgram(ok ? EXIT_SUCCESS : EXIT_FAILURE);
 }
+/* *INDENT-OFF* */
+VERSION_COMMON()
+/* *INDENT-ON* */
 
 int
 main(int argc, char *argv[])
 {
-    static const char *help[] =
+    static NCURSES_CONST char *help[] =
     {
 	"Commands:",
 	"  q,^Q,ESC       - quit this program",
@@ -286,9 +302,10 @@ main(int argc, char *argv[])
 	"  s              - use entered count for halfdelay() parameter",
 	"                 - if no entered count, stop nodelay()",
 	"  <space>        - begin nodelay()",
-	0
+	NULL
     };
 
+    int ch;
     int i;
     int my_delay = 0;
     WINDOW *my_pad;
@@ -302,9 +319,9 @@ main(int argc, char *argv[])
 
     setlocale(LC_ALL, "");
 
-    while ((i = getopt(argc, argv, "cinstT:")) != -1) {
-	switch (i) {
-	case 'c':
+    while ((ch = getopt(argc, argv, OPTS_COMMON "CD:inst")) != -1) {
+	switch (ch) {
+	case 'C':
 	    try_color = TRUE;
 	    break;
 	case 'i':
@@ -319,12 +336,12 @@ main(int argc, char *argv[])
 	    single_step = TRUE;
 	    break;
 #ifdef TRACE
-	case 'T':
+	case 'D':
 	    {
-		char *next = 0;
+		char *next = NULL;
 		int tvalue = (int) strtol(optarg, &next, 0);
-		if (tvalue < 0 || (next != 0 && *next != 0))
-		    usage();
+		if (tvalue < 0 || (next != NULL && *next != 0))
+		    usage(FALSE);
 		curses_trace((unsigned) tvalue);
 	    }
 	    break;
@@ -332,12 +349,16 @@ main(int argc, char *argv[])
 	    curses_trace(TRACE_CALLS);
 	    break;
 #endif
+	case OPTS_VERSION:
+	    show_version(argv);
+	    ExitProgram(EXIT_SUCCESS);
 	default:
-	    usage();
+	    usage(ch == OPTS_USAGE);
+	    /* NOTREACHED */
 	}
     }
     if (optind + 1 != argc)
-	usage();
+	usage(FALSE);
 
     InitAndCatch(initscr(), ignore_sigs ? SIG_IGN : finish);
     keypad(stdscr, TRUE);	/* enable keyboard mapping */
@@ -362,7 +383,8 @@ main(int argc, char *argv[])
      * Do this after starting color, otherwise the pad's background will be
      * uncolored after the ncurses 6.1.20181208 fixes.
      */
-    my_pad = read_file(fname = argv[optind]);
+    global_pad =
+	my_pad = read_file(fname = argv[optind]);
 
     my_row = 0;
     while (!done) {
@@ -511,7 +533,7 @@ main(int argc, char *argv[])
 	    beep();
 	    break;
 	}
-	if (c >= KEY_MIN || (c > 0 && !isdigit(c))) {
+	if (c >= KEY_MIN || (c > 0 && !isdigit(UChar(c)))) {
 	    got_number = FALSE;
 	    value = 0;
 	}
