@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2018-2020,2021 Thomas E. Dickey                                *
+ * Copyright 2018-2023,2024 Thomas E. Dickey                                *
  * Copyright 2017,2018 Free Software Foundation, Inc.                       *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -27,7 +27,7 @@
  * authorization.                                                           *
  ****************************************************************************/
 /*
- * $Id: picsmap.c,v 1.139 2021/05/08 15:56:05 tom Exp $
+ * $Id: picsmap.c,v 1.153 2024/12/07 22:48:03 tom Exp $
  *
  * Author: Thomas E. Dickey
  *
@@ -53,13 +53,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#if HAVE_STDINT_H
-#include <stdint.h>
-#define my_intptr_t	intptr_t
-#else
-#define my_intptr_t	long
-#endif
-
 #if HAVE_TSEARCH
 #include <search.h>
 #endif
@@ -116,7 +109,7 @@ static void logmsg2(const char *fmt, ...) GCC_PRINTFLIKE(1, 2);
 static void warning(const char *fmt, ...) GCC_PRINTFLIKE(1, 2);
 static int gather_c_values(int);
 
-static FILE *logfp = 0;
+static FILE *logfp = NULL;
 static double aspect_ratio = 0.6;
 static bool in_curses = FALSE;
 static bool debugging = FALSE;
@@ -145,7 +138,7 @@ static bool use_extended_colors = FALSE;
 static void
 logmsg(const char *fmt, ...)
 {
-    if (logfp != 0) {
+    if (logfp != NULL) {
 	va_list ap;
 	va_start(ap, fmt);
 	vfprintf(logfp, fmt, ap);
@@ -158,7 +151,7 @@ logmsg(const char *fmt, ...)
 static void
 logmsg2(const char *fmt, ...)
 {
-    if (logfp != 0) {
+    if (logfp != NULL) {
 	va_list ap;
 	va_start(ap, fmt);
 	vfprintf(logfp, fmt, ap);
@@ -170,7 +163,7 @@ logmsg2(const char *fmt, ...)
 static void
 close_log(void)
 {
-    if (logfp != 0) {
+    if (logfp != NULL) {
 	logmsg("Allocations:");
 	logmsg("%8ld file", (long) how_much.file);
 	logmsg("%8ld name", (long) how_much.name);
@@ -181,7 +174,7 @@ close_log(void)
 	logmsg("%8ld cell", (long) how_much.cell);
 	logmsg("%8ld window", LINES * COLS * (long) sizeof(NCURSES_CH_T));
 	fclose(logfp);
-	logfp = 0;
+	logfp = NULL;
     }
 }
 
@@ -206,7 +199,7 @@ failed(const char *msg)
 static void
 warning(const char *fmt, ...)
 {
-    if (logfp != 0) {
+    if (logfp != NULL) {
 	va_list ap;
 	va_start(ap, fmt);
 	vfprintf(logfp, fmt, ap);
@@ -226,7 +219,7 @@ warning(const char *fmt, ...)
 static void
 free_data(char **data)
 {
-    if (data != 0) {
+    if (data != NULL) {
 	free(data[0]);
 	free(data);
     }
@@ -235,12 +228,12 @@ free_data(char **data)
 static PICS_HEAD *
 free_pics_head(PICS_HEAD * pics)
 {
-    if (pics != 0) {
+    if (pics != NULL) {
 	free(pics->fgcol);
 	free(pics->cells);
 	free(pics->name);
 	free(pics);
-	pics = 0;
+	pics = NULL;
     }
     return pics;
 }
@@ -312,14 +305,14 @@ gather_c_values(int fg)
     reading_ncols[next].count = 0;
 
     check_c_values(__LINE__);
-    if ((ft = tfind(I2P(next), &reading_ntree, compare_c_values)) != 0) {
+    if ((ft = tfind(I2P(next), &reading_ntree, compare_c_values)) != NULL) {
 	found = P2I(*ft);
     } else {
 	if (reading_last + 2 >= reading_size) {
 	    int more = ((MAX(reading_last, reading_size) + 2) * 3) / 2;
 	    int last = reading_last + 1;
 	    FG_NODE *p = typeRealloc(FG_NODE, more, reading_ncols);
-	    if (p == 0)
+	    if (p == NULL)
 		goto done;
 
 	    reading_size = more;
@@ -330,7 +323,7 @@ gather_c_values(int fg)
 	}
 	++reading_last;
 	how_much.pair += sizeof(FG_NODE);
-	if ((ft = tsearch(I2P(next), &reading_ntree, compare_c_values)) != 0) {
+	if ((ft = tsearch(I2P(next), &reading_ntree, compare_c_values)) != NULL) {
 	    found = P2I(*ft);
 	    if (found != next)
 		logmsg("OOPS expected slot %d, got %d", next, found);
@@ -377,24 +370,24 @@ finish_c_values(PICS_HEAD * head)
 
     reading_last = 0;
     reading_size = 0;
-    reading_ncols = 0;
+    reading_ncols = NULL;
 }
 
 static void
 dispose_c_values(void)
 {
 #if HAVE_TSEARCH
-    if (reading_ntree != 0) {
+    if (reading_ntree != NULL) {
 	int n;
 	for (n = 0; n < reading_last; ++n) {
 	    tdelete(I2P(n), &reading_ntree, compare_c_values);
 	}
-	reading_ntree = 0;
+	reading_ntree = NULL;
     }
 #endif
-    if (reading_ncols != 0) {
+    if (reading_ncols != NULL) {
 	free(reading_ncols);
-	reading_ncols = 0;
+	reading_ncols = NULL;
     }
     reading_last = 0;
     reading_size = 0;
@@ -420,7 +413,7 @@ is_file(const char *filename, struct stat *sb)
 static char **
 read_file(const char *filename)
 {
-    char **result = 0;
+    char **result = NULL;
     struct stat sb;
 
     if (!quiet) {
@@ -437,9 +430,9 @@ read_file(const char *filename)
 	result = typeCalloc(char *, size + 1);
 	how_much.file += ((size + 1) * 2);
 
-	if (blob != 0 && result != 0) {
+	if (blob != NULL && result != NULL) {
 	    FILE *fp = fopen(filename, "r");
-	    if (fp != 0) {
+	    if (fp != NULL) {
 		logmsg("opened %s", filename);
 
 		if (fread(blob, sizeof(char), size, fp) == size) {
@@ -464,7 +457,7 @@ read_file(const char *filename)
 			    result[k++] = blob + j;
 			}
 		    }
-		    result[k] = 0;
+		    result[k] = NULL;
 		    if (k && !binary) {
 			debugmsg2("[%5d] %s\n", k, result[k - 1]);
 		    }
@@ -478,7 +471,7 @@ read_file(const char *filename)
 	    debugmsg("...file is empty");
 	    free(blob);
 	    free(result);
-	    result = 0;
+	    result = NULL;
 	} else if (binary) {
 	    debugmsg("...file is non-text");
 	}
@@ -487,27 +480,28 @@ read_file(const char *filename)
 }
 
 static void
-usage(void)
+usage(int ok)
 {
     static const char *msg[] =
     {
 	"Usage: picsmap [options] [imagefile [...]]"
 	,"Read/display one or more xbm/xpm files (possibly use \"convert\")"
 	,""
+	,USAGE_COMMON
 	,"Options:"
-	,"  -a ratio     aspect-ratio correction for ImageMagick"
+	," -a ratio aspect-ratio correction for ImageMagick"
 #if HAVE_USE_DEFAULT_COLORS
-	,"  -d           invoke use_default_colors"
+	," -d       invoke use_default_colors"
 #endif
-	,"  -L           add debugging information to logfile"
-	,"  -l logfile   write informational messages to logfile"
-	,"  -p palette   color-palette file (default \"$TERM.dat\")"
-	,"  -q           less verbose"
-	,"  -r rgb-path  xpm uses X rgb color-names (default \"" RGB_PATH "\")"
-	,"  -s SECS      pause for SECS seconds after display vs getch"
+	," -L       add debugging information to logfile"
+	," -F FILE  write informational messages to FILE"
+	," -p FILE  color-palette file (default \"$TERM.dat\")"
+	," -q       less verbose"
+	," -r FILE  xpm uses X rgb color-names in FILE (default \"" RGB_PATH "\")"
+	," -s SECS  pause for SECS seconds after display vs getch"
 #if USE_EXTENDED_COLORS
-	,"  -x [pc]      use extension (p=extended-pairs, c=extended-colors)"
-	,"               Either/both extension may be given"
+	," -x [pc]  use extension (p=extended-pairs, c=extended-colors)"
+	,"          Either/both extension may be given"
 #endif
     };
     size_t n;
@@ -517,7 +511,7 @@ usage(void)
     fflush(stdout);
     for (n = 0; n < SIZEOF(msg); n++)
 	fprintf(stderr, "%s\n", msg[n]);
-    cleanup(EXIT_FAILURE);
+    cleanup(ok ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 static void
@@ -541,7 +535,7 @@ giveup(const char *fmt, ...)
 	fflush(logfp);
     }
 
-    usage();
+    usage(FALSE);
 }
 
 /*
@@ -554,20 +548,20 @@ static char **
 read_palette(const char *filename)
 {
     static const char *data_dir = DATA_DIR;
-    char **result = 0;
+    char **result = NULL;
     size_t last = strlen(filename);
     size_t need = (strlen(data_dir) + 20 + last);
     char *full_name = malloc(need);
     char *s;
     struct stat sb;
 
-    if (full_name != 0) {
+    if (full_name != NULL) {
 	int tries;
 	for (tries = 0; tries < 8; ++tries) {
 
 	    *(s = full_name) = '\0';
 	    if (tries & 1) {
-		if (strchr(filename, '/') == 0) {
+		if (strchr(filename, '/') == NULL) {
 		    _nc_SPRINTF(full_name, _nc_SLIMIT(need) "%s/", data_dir);
 		} else {
 		    continue;
@@ -580,7 +574,7 @@ read_palette(const char *filename)
 	    _nc_STRCAT(full_name, filename, need);
 	    if (tries & 4) {
 		char *t = s;
-		char *tc;
+		const char *tc;
 		int num;
 		char chr;
 		int found = 0;
@@ -588,7 +582,7 @@ read_palette(const char *filename)
 		    if (*t == '-') {
 			if (sscanf(t, "-%d%c", &num, &chr) == 2 &&
 			    chr == 'c' &&
-			    (tc = strchr(t, chr)) != 0 &&
+			    (tc = strchr(t, chr)) != NULL &&
 			    !(strncmp) (tc, "color", 5)) {
 			    found = 1;
 			}
@@ -628,7 +622,7 @@ read_palette(const char *filename)
 static void
 init_palette(const char *palette_file)
 {
-    if (palette_file != 0) {
+    if (palette_file != NULL) {
 	char **data = read_palette(palette_file);
 
 	all_colors = typeMalloc(RGB_DATA, (unsigned) COLORS);
@@ -647,12 +641,12 @@ init_palette(const char *palette_file)
 #else
 	memset(all_colors, 0, sizeof(RGB_DATA) * (size_t) COLORS);
 #endif
-	if (data != 0) {
+	if (data != NULL) {
 	    int n;
 	    int red, green, blue;
 	    int scale = MaxSCALE;
 	    int c;
-	    for (n = 0; data[n] != 0; ++n) {
+	    for (n = 0; data[n] != NULL; ++n) {
 		if (sscanf(data[n], "scale:%d", &c) == 1) {
 		    scale = c;
 		} else if (sscanf(data[n], "%d:%d %d %d",
@@ -683,9 +677,9 @@ init_palette(const char *palette_file)
 	}
 
 	if ((power2 != COLORS) || ((shift % 3) != 0)) {
-	    if (all_colors == 0) {
+	    if (all_colors == NULL) {
 		init_palette(getenv("TERM"));
-		if (all_colors == 0) {
+		if (all_colors == NULL) {
 		    giveup("With %d colors, you need a palette-file", COLORS);
 		}
 	    }
@@ -710,7 +704,7 @@ map_color(int value)
 	int green = (value & 0x00ff00) >> 8;
 	int blue  = (value & 0x0000ff) >> 0;
 
-	if (all_colors != 0) {
+	if (all_colors != NULL) {
 #define Diff2(n,m) ((m) - all_colors[n].m) * ((m) - all_colors[n].m)
 #define Diff2S(n) Diff2(n,red) + Diff2(n,green) + Diff2(n,blue)
 	    int d2 = Diff2S(0);
@@ -797,6 +791,7 @@ match_c(const char *source, const char *pattern, ...)
     int ch;
     int *ip;
     char *cp;
+    float *fp;
     long lv;
 
     va_start(ap, pattern);
@@ -810,10 +805,13 @@ match_c(const char *source, const char *pattern, ...)
 	    continue;
 	}
 	/* %c, %d, %s are like sscanf except for special treatment of blanks */
-	if (ch == '%' && *pattern != '\0' && strchr("cdnsx", *pattern)) {
+	if (ch == '%' && *pattern != '\0' && strchr("%cdnfsx", *pattern)) {
 	    bool found = FALSE;
 	    ch = *pattern++;
 	    switch (ch) {
+	    case '%':
+		source++;
+		break;
 	    case 'c':
 		cp = va_arg(ap, char *);
 		do {
@@ -825,13 +823,36 @@ match_c(const char *source, const char *pattern, ...)
 		limit = -1;
 		ip = va_arg(ap, int *);
 		lv = strtol(source, &cp, ch == 'd' ? 10 : 16);
-		if (cp != 0 && cp != source) {
+		if (cp != NULL && cp != source) {
 		    *ip = (int) lv;
 		    source = cp;
 		} else {
 		    goto finish;
 		}
 		break;
+	    case 'f':
+		/* floating point for pixels... */
+		fp = va_arg(ap, float *);
+		lv = strtol(source, &cp, 10);
+		if (cp == NULL || cp == source)
+		    goto finish;
+		*fp = (float) lv;
+		source = cp;
+		if (*source == '.') {
+		    lv = strtol(++source, &cp, 10);
+		    if (cp == NULL || cp == source)
+			goto finish;
+		    {
+			float scale = 1.0f;
+			int digits = (int) (cp - source);
+			while (digits-- > 0) {
+			    scale *= 10.0f;
+			}
+			*fp += (float) lv / scale;
+		    }
+		    source = cp;
+		}
+		break;
 	    case 'n':
 		/* not really sscanf... */
 		limit = *va_arg(ap, int *);
@@ -886,7 +907,7 @@ match_colors(const char *source, int cpp, char *arg1, char *arg2, char *arg3)
 	    s += cpp;
 	    while (*s++ == '\t') {
 		char *t;
-		for (t = arg2; (*s != '\0') && strchr("\t\"", *s) == 0;) {
+		for (t = arg2; (*s != '\0') && strchr("\t\"", *s) == NULL;) {
 		    if (*s == ' ') {
 			s = skip_cs(s);
 			break;
@@ -894,7 +915,7 @@ match_colors(const char *source, int cpp, char *arg1, char *arg2, char *arg3)
 		    *t++ = *s++;
 		    *t = '\0';
 		}
-		for (t = arg3; (*s != '\0') && strchr("\t\"", *s) == 0;) {
+		for (t = arg3; (*s != '\0') && strchr("\t\"", *s) == NULL;) {
 		    *t++ = *s++;
 		    *t = '\0';
 		}
@@ -917,16 +938,17 @@ parse_rgb(char **data)
     char *s, *t;
     size_t item = 0;
     size_t need;
-    RGB_NAME *result = 0;
+    RGB_NAME *result = NULL;
 
-    for (need = 0; data[need] != 0; ++need) ;
+    for (need = 0; data[need] != NULL; ++need) ;
 
     result = typeCalloc(RGB_NAME, need + 2);
     how_much.name += (sizeof(RGB_NAME) * (need + 2));
 
-    for (n = 0; data[n] != 0; ++n) {
+    for (n = 0; data[n] != NULL; ++n) {
 	if (strlen(t = data[n]) >= sizeof(buf) - 1)
 	    continue;
+	t = strcpy(buf, t);
 	if (*(s = skip_s(t)) == '!')
 	    continue;
 
@@ -971,10 +993,10 @@ CaselessCmp(const char *a, const char *b)
 static RGB_NAME *
 lookup_rgb(const char *name)
 {
-    RGB_NAME *result = 0;
-    if (rgb_table != 0) {
+    RGB_NAME *result = NULL;
+    if (rgb_table != NULL) {
 	int n;
-	for (n = 0; rgb_table[n].name != 0; ++n) {
+	for (n = 0; rgb_table[n].name != NULL; ++n) {
 	    if (!CaselessCmp(name, rgb_table[n].name)) {
 		result = &rgb_table[n];
 		break;
@@ -1007,7 +1029,7 @@ parse_xbm(char **data)
     gather_c_values(0);
     gather_c_values(0xffffff);
 
-    for (n = 0; data[n] != 0; ++n) {
+    for (n = 0; data[n] != NULL; ++n) {
 	if (strlen(s = data[n]) >= sizeof(buf) - 1)
 	    continue;
 	switch (state) {
@@ -1015,10 +1037,10 @@ parse_xbm(char **data)
 	case 1:
 	case 2:
 	    if (sscanf(s, "#define %1024s %d%c", buf, &num, &ch) >= 2) {
-		if ((t = strstr(buf, "_width")) != 0) {
+		if ((t = strstr(buf, "_width")) != NULL) {
 		    state |= 1;
 		    result->wide = (short) bytes_of(num);
-		} else if ((t = strstr(buf, "_height")) != 0) {
+		} else if ((t = strstr(buf, "_height")) != NULL) {
 		    state |= 2;
 		    result->high = (short) num;
 		} else {
@@ -1045,7 +1067,7 @@ parse_xbm(char **data)
 		result->cells = typeCalloc(PICS_CELL, cells);
 		how_much.cell += (sizeof(PICS_CELL) * cells);
 
-		if ((s = strchr(s, L_CURLY)) == 0)
+		if ((s = strchr(s, L_CURLY)) == NULL)
 		    break;
 		++s;
 	    } else {
@@ -1125,16 +1147,17 @@ parse_xpm(char **data)
     char arg1[BUFSIZ];
     char arg2[BUFSIZ];
     char arg3[BUFSIZ];
-    char **list = 0;
+    char **list = NULL;
 
     debugmsg("called parse_xpm");
 
     result = typeCalloc(PICS_HEAD, 1);
     how_much.head += sizeof(PICS_HEAD);
 
-    for (n = 0; data[n] != 0; ++n) {
+    for (n = 0; data[n] != NULL; ++n) {
 	if (strlen(s = data[n]) >= sizeof(buf) - 1)
 	    continue;
+	s = strcpy(buf, s);
 	switch (state) {
 	case 0:
 	    if (match_c(s, " /* XPM */ ")) {
@@ -1178,7 +1201,7 @@ parse_xpm(char **data)
 	    num_colors++;
 	    free(list[reading_last]);
 	    list[reading_last] = strdup(arg1);
-	    if ((by_name = lookup_rgb(arg3)) != 0) {
+	    if ((by_name = lookup_rgb(arg3)) != NULL) {
 		found = gather_c_values(by_name->value);
 	    } else if (*arg3 == '#') {
 		char *rgb = arg3 + 1;
@@ -1204,7 +1227,7 @@ parse_xpm(char **data)
 	    if (num_colors >= result->colors) {
 		finish_c_values(result);
 		state = 4;
-		if (list[0] == 0)
+		if (list[0] == NULL)
 		    list[0] = strdup("\033");
 	    }
 	    break;
@@ -1216,7 +1239,7 @@ parse_xpm(char **data)
 
 		    /* FIXME - factor out */
 		    for (c = 0; c < result->colors; ++c) {
-			if (list[c] == 0) {
+			if (list[c] == NULL) {
 			    /* should not happen... */
 			    continue;
 			}
@@ -1291,8 +1314,8 @@ parse_img(const char *filename)
     result = typeCalloc(PICS_HEAD, 1);
     how_much.head += sizeof(PICS_HEAD);
 
-    if ((pp = popen(cmd, "r")) != 0) {
-	if (fgets(buffer, sizeof(buffer), pp) != 0) {
+    if ((pp = popen(cmd, "r")) != NULL) {
+	if (fgets(buffer, sizeof(buffer), pp) != NULL) {
 	    size_t n = strlen(filename);
 	    debugmsg2("...read %s", buffer);
 	    if (strlen(buffer) > n &&
@@ -1318,12 +1341,12 @@ parse_img(const char *filename)
 	_nc_STRCAT(cmd, " 2>/dev/null", need);
 
     logmsg("...opening pipe to %s", cmd);
-    if ((pp = popen(cmd, "r")) != 0) {
+    if ((pp = popen(cmd, "r")) != NULL) {
 	int count = 0;
 	int col = 0;
 	int row = 0;
 	int len = 0;
-	while (fgets(buffer, sizeof(buffer), pp) != 0) {
+	while (fgets(buffer, sizeof(buffer), pp) != NULL) {
 	    debugmsg2("[%5d] %s", count + 1, buffer);
 	    if (strlen(buffer) > 160) {		/* 80 columns would be enough */
 		okay = FALSE;
@@ -1346,31 +1369,61 @@ parse_img(const char *filename)
 		    break;
 		}
 	    } else {
-		/* subsequent lines begin "col,row: (r,g,b,a) #RGB" */
+		/*
+		 * subsequent lines begin "col,row: (r,g,b,a) #RGB".
+		 * Those r/g/b could be integers (0..255) or float-percentages.
+		 */
 		int r, g, b, nocolor;
+		float rf, gf, bf;
 		unsigned check;
-		char *t;
-		char *s = t = strchr(buffer, '#');
+		char *s = strchr(buffer, '#');
+		const char *t = s;
+		bool matched = FALSE;
 
-		if (s != 0) {
+		if (s != NULL) {
 		    /* after the "#RGB", there are differences - just ignore */
 		    while (*s != '\0' && !isspace(UChar(*s)))
 			++s;
 		    *++s = '\0';
 		}
+
 		if (match_c(buffer,
 			    "%d,%d: (%d,%d,%d,%d) #%x ",
 			    &col, &row,
 			    &r, &g, &b, &nocolor,
 			    &check)) {
-		    int which, c;
+		    matched = TRUE;
+		} else if (match_c(buffer,
+				   "%d,%d: (%f%%,%f%%,%f%%,%d) #%x ",
+				   &col, &row,
+				   &rf, &gf, &bf, &nocolor,
+				   &check) ||
+			   match_c(buffer,
+				   "%d,%d: (%f%%,%f%%,%f%%) #%x ",
+				   &col, &row,
+				   &rf, &gf, &bf,
+				   &check)) {
+		    matched = TRUE;
 
-		    if ((s - t) > 8)	/* 6 hex digits vs 8 */
-			check /= 256;
-		    if (r > MaxRGB ||
-			g > MaxRGB ||
-			b > MaxRGB ||
-			check != (unsigned) ((r << 16) | (g << 8) | b)) {
+#define fp_fix(n) (int) (MaxRGB * (((n) > 100.0 ? 100.0 : (n)) / 100.0))
+
+		    r = fp_fix(rf);
+		    g = fp_fix(gf);
+		    b = fp_fix(bf);
+		}
+		if ((s - t) > 8)	/* 6 hex digits vs 8 */
+		    check /= 256;
+		if (matched) {
+		    int which, c;
+		    int want_r = (check >> 16) & 0xff;
+		    int want_g = (check >> 8) & 0xff;
+		    int want_b = (check >> 0) & 0xff;
+
+#define fp_err(tst,ref) ((tst > MaxRGB) || ((tst - ref)*(tst - ref)) > 4)
+
+		    if (fp_err(r, want_r) ||
+			fp_err(g, want_g) ||
+			fp_err(b, want_b)) {
 			okay = FALSE;
 			break;
 		    }
@@ -1419,11 +1472,11 @@ static PICS_HEAD *
 read_picture(const char *filename, char **data)
 {
     PICS_HEAD *pics;
-    if ((pics = parse_xbm(data)) == 0) {
+    if ((pics = parse_xbm(data)) == NULL) {
 	dispose_c_values();
-	if ((pics = parse_xpm(data)) == 0) {
+	if ((pics = parse_xpm(data)) == NULL) {
 	    dispose_c_values();
-	    if ((pics = parse_img(filename)) == 0) {
+	    if ((pics = parse_img(filename)) == NULL) {
 		dispose_c_values();
 		free_data(data);
 		warning("unexpected file-format for \"%s\"", filename);
@@ -1441,7 +1494,7 @@ read_picture(const char *filename, char **data)
 #define fg_color(pics,n) (pics->fgcol[n].fgcol)
 
 static void
-dump_picture(PICS_HEAD * pics)
+dump_picture(const PICS_HEAD * pics)
 {
     int y, x;
 
@@ -1470,6 +1523,7 @@ init_display(const char *palette_path, int opt_d)
     (void) opt_d;
     if (isatty(fileno(stdout))) {
 	in_curses = TRUE;
+	setlocale(LC_ALL, "");
 	initscr();
 	cbreak();
 	noecho();
@@ -1584,7 +1638,7 @@ report_colors(PICS_HEAD * pics)
     int total;
     char buffer[256];
 
-    if (logfp == 0)
+    if (logfp == NULL)
 	return;
 
     qsort(pics->fgcol, (size_t) pics->colors, sizeof(FG_NODE), compare_fg_counts);
@@ -1665,24 +1719,27 @@ report_colors(PICS_HEAD * pics)
 	}
     }
 }
+/* *INDENT-OFF* */
+VERSION_COMMON()
+/* *INDENT-ON* */
 
 int
 main(int argc, char *argv[])
 {
-    int n;
+    int ch;
     int opt_d = FALSE;
     char ignore_ch;
-    const char *palette_path = 0;
+    const char *palette_path = NULL;
     const char *rgb_path = RGB_PATH;
 
-    while ((n = getopt(argc, argv, "a:dLl:p:qr:s:x:")) != -1) {
-	switch (n) {
+    while ((ch = getopt(argc, argv, OPTS_COMMON "a:dLF:p:qr:s:x:")) != -1) {
+	switch (ch) {
 	case 'a':
 	    if (sscanf(optarg, "%lf%c", &aspect_ratio, &ignore_ch) != 1
 		|| aspect_ratio < 0.1
 		|| aspect_ratio > 10.) {
 		fprintf(stderr, "Expected a number in [0.1 to 10.]: %s\n", optarg);
-		usage();
+		usage(FALSE);
 	    }
 	    break;
 #if HAVE_USE_DEFAULT_COLORS
@@ -1693,8 +1750,8 @@ main(int argc, char *argv[])
 	case 'L':
 	    debugging = TRUE;
 	    break;
-	case 'l':
-	    if ((logfp = fopen(optarg, "a")) == 0)
+	case 'F':
+	    if ((logfp = fopen(optarg, "a")) == NULL)
 		failed(optarg);
 	    break;
 	case 'p':
@@ -1712,7 +1769,7 @@ main(int argc, char *argv[])
 #if USE_EXTENDED_COLORS
 	case 'x':
 	    {
-		char *s = optarg;
+		const char *s = optarg;
 		while (*s) {
 		    switch (*s++) {
 		    case 'p':
@@ -1722,21 +1779,25 @@ main(int argc, char *argv[])
 			use_extended_colors = TRUE;
 			break;
 		    default:
-			usage();
+			usage(FALSE);
 			break;
 		    }
 		}
 	    }
 	    break;
 #endif
+	case OPTS_VERSION:
+	    show_version(argv);
+	    ExitProgram(EXIT_SUCCESS);
 	default:
-	    usage();
-	    break;
+	    usage(ch == OPTS_USAGE);
+	    /* NOTREACHED */
 	}
     }
 
     if (optind < argc) {
 	char **rgb_data = read_file(rgb_path);
+	int n;
 
 	if (rgb_data)
 	    rgb_table = parse_rgb(rgb_data);
@@ -1749,11 +1810,11 @@ main(int argc, char *argv[])
 	    PICS_HEAD *pics;
 	    char **data = read_file(argv[n]);
 
-	    if (data == 0) {
+	    if (data == NULL) {
 		warning("cannot read \"%s\"", argv[n]);
 		continue;
 	    }
-	    if ((pics = read_picture(argv[n], data)) != 0) {
+	    if ((pics = read_picture(argv[n], data)) != NULL) {
 		if (in_curses) {
 		    show_picture(pics);
 		} else {
@@ -1769,7 +1830,7 @@ main(int argc, char *argv[])
 	free(rgb_table);
 	free(all_colors);
     } else {
-	usage();
+	usage(FALSE);
     }
 
     cleanup(EXIT_SUCCESS);
