@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2018-2019,2020 Thomas E. Dickey                                *
+ * Copyright 2018-2022,2024 Thomas E. Dickey                                *
  * Copyright 2006-2017,2018 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -27,7 +27,7 @@
  * authorization.                                                           *
  ****************************************************************************/
 /*
- * $Id: savescreen.c,v 1.57 2020/02/02 23:34:34 tom Exp $
+ * $Id: savescreen.c,v 1.67 2024/12/07 22:46:42 tom Exp $
  *
  * Demonstrate save/restore functions from the curses library.
  * Thomas Dickey - 2007/7/14
@@ -86,17 +86,17 @@ setup_next(void)
 static void
 cleanup(char *files[])
 {
-    int n;
-
     if (!keep_dumps) {
-	for (n = 0; files[n] != 0; ++n) {
+	int n;
+
+	for (n = 0; files[n] != NULL; ++n) {
 	    unlink(files[n]);
 	}
     }
 }
 
 static int
-load_screen(char *filename)
+load_screen(NCURSES_CONST char *filename)
 {
     int result;
 
@@ -169,12 +169,11 @@ dump_screen(char **files, int color, int which, int last, bool use_colors)
 {
 #if USE_WIDEC_SUPPORT
     cchar_t mycc;
-    int myxx;
 #endif
-    char *filename = files[which];
+    NCURSES_CONST char *filename = files[which];
     bool dumped = FALSE;
 
-    if (filename != 0) {
+    if (filename != NULL) {
 	dumped = TRUE;
 	show_what(color, ++which, last);
 	if (scr_dump(filename) == ERR) {
@@ -198,7 +197,7 @@ dump_screen(char **files, int color, int which, int last, bool use_colors)
 		    wmove(stdscr, cy, cx);
 #if USE_WIDEC_SUPPORT
 		    if (win_wch(curscr, &mycc) != ERR) {
-			myxx = wcwidth(BaseChar(mycc));
+			int myxx = wcwidth(BaseChar(mycc));
 			if (myxx > 0) {
 			    wadd_wchnstr(stdscr, &mycc, 1);
 			    cx += (myxx - 1);
@@ -217,7 +216,7 @@ dump_screen(char **files, int color, int which, int last, bool use_colors)
 static void
 editor_help(void)
 {
-    static const char *msgs[] =
+    static NCURSES_CONST char *msgs[] =
     {
 	"You are now in the screen-editor, which allows you to make some",
 	"lines on the screen, as well as save copies of the screen to a",
@@ -231,7 +230,7 @@ editor_help(void)
 	"   a           toggle between '#' and graphic symbol for drawing",
 	"   c           change color drawn by line to next in palette",
 	"   h,j,k,l or arrows to move around the screen, drawing",
-	0
+	NULL
     };
     popup_msg(stdscr, msgs);
 }
@@ -239,7 +238,7 @@ editor_help(void)
 static void
 replay_help(void)
 {
-    static const char *msgs[] =
+    static NCURSES_CONST char *msgs[] =
     {
 	"You are now in the screen-loader, which allows you to view",
 	"the dumped/restored screens.",
@@ -248,30 +247,34 @@ replay_help(void)
 	"   q           quit",
 	"   <space>     load the next screen",
 	"   <backspace> load the previous screen",
-	0
+	NULL
     };
     popup_msg(stdscr, msgs);
 }
 
 static void
-usage(void)
+usage(int ok)
 {
     static const char *msg[] =
     {
-	"Usage: savescreen [-r] files",
-	"",
-	"Options:",
-	" -f file  fill/initialize screen using text from this file",
-	" -i       use scr_init/scr_restore rather than scr_set",
-	" -k       keep the restored dump-files rather than removing them",
-	" -r       replay the screen-dump files"
+	"Usage: savescreen [-r] files"
+	,""
+	,USAGE_COMMON
+	,"Options:"
+	," -f FILE  fill/initialize screen using text from this file"
+	," -i       use scr_init/scr_restore rather than scr_set"
+	," -k       keep the restored dump-files rather than removing them"
+	," -r       replay the screen-dump files"
     };
     unsigned n;
     for (n = 0; n < SIZEOF(msg); ++n) {
 	fprintf(stderr, "%s\n", msg[n]);
     }
-    ExitProgram(EXIT_FAILURE);
+    ExitProgram(ok ? EXIT_SUCCESS : EXIT_FAILURE);
 }
+/* *INDENT-OFF* */
+VERSION_COMMON()
+/* *INDENT-ON* */
 
 int
 main(int argc, char *argv[])
@@ -283,21 +286,16 @@ main(int argc, char *argv[])
     bool replaying = FALSE;
     bool done = FALSE;
     char **files;
-    char *fill_by = 0;
+    NCURSES_CONST char *fill_by = NULL;
 #if USE_WIDEC_SUPPORT
-    int r, g, b;
-    int cube = 0;
-    int cube0 = 16;
-    int cube1;
     cchar_t mycc;
     static const wchar_t mywc[2] =
     {L'#', 0};
-    bool using_rgb = FALSE;
 #endif
 
     setlocale(LC_ALL, "");
 
-    while ((ch = getopt(argc, argv, "f:ikr")) != -1) {
+    while ((ch = getopt(argc, argv, OPTS_COMMON "f:ikr")) != -1) {
 	switch (ch) {
 	case 'f':
 	    fill_by = optarg;
@@ -311,9 +309,12 @@ main(int argc, char *argv[])
 	case 'r':
 	    replaying = TRUE;
 	    break;
+	case OPTS_VERSION:
+	    show_version(argv);
+	    ExitProgram(EXIT_SUCCESS);
 	default:
-	    usage();
-	    break;
+	    usage(ch == OPTS_USAGE);
+	    /* NOTREACHED */
 	}
     }
 
@@ -332,6 +333,9 @@ main(int argc, char *argv[])
     curs_set(0);
 
     if (has_colors() && (start_color() == OK) && COLORS >= MAX_ANSI) {
+#if USE_WIDEC_SUPPORT
+	bool using_rgb = FALSE;
+#endif
 	static const struct {
 	    int fg, bg;
 	} table[MAX_ANSI] = {
@@ -362,7 +366,7 @@ main(int argc, char *argv[])
 	 * After that, use color pairs for constructing a test-pattern, e.g.,
 	 * imitating xterm's scripts.
 	 */
-	if (fill_by == 0) {
+	if (fill_by == NULL) {
 	    if (COLORS <= 256) {
 		for (n = 0; n < COLORS; ++n)
 		    init_pair((short) (n + MAX_ANSI), (short) n, (short) n);
@@ -378,13 +382,13 @@ main(int argc, char *argv[])
 		    int r_delta = (r_max / cols);
 		    int g_delta = (g_max / cols);
 		    int row = 0;
+		    int b = 0;
 
-		    b = 0;
 		    using_rgb = TRUE;
 		    while (row++ < rows) {
 			int col = 0;
-			r = 0;
-			g = g_max;
+			int r = 0;
+			int g = g_max;
 			while (col++ < cols) {
 			    int color = (((r * (g_max + 1)) + g) * (b_max + 1)
 					 + b + MAX_ANSI);
@@ -403,7 +407,10 @@ main(int argc, char *argv[])
 	    }
 #endif
 	}
-	if ((fill_by == 0) && !replaying) {
+	if ((fill_by == NULL) && !replaying) {
+#if USE_WIDEC_SUPPORT
+	    int cube = 0;
+#endif
 	    /*
 	     * Originally (before wide-characters) ncurses supported 16 colors.
 	     */
@@ -428,8 +435,9 @@ main(int argc, char *argv[])
 		cube = 6;
 	    }
 	    if (cube != 0) {
-		cube0 = 16;
-		cube1 = cube0 + (cube * cube * cube);
+		int r, g, b;
+		int cube0 = 16;
+		int cube1 = cube0 + (cube * cube * cube);
 
 		addch('\n');
 		printw("Color cube, %dx%dx%d:\n", cube, cube, cube);
@@ -460,7 +468,6 @@ main(int argc, char *argv[])
 		int cols = COLS - 1;
 		int row = 0;
 
-		b = 0;
 		pair = MAX_ANSI;
 		while (row++ < rows) {
 		    int col = 0;
@@ -477,9 +484,9 @@ main(int argc, char *argv[])
 	}
     }
 
-    if (fill_by != 0) {
+    if (fill_by != NULL) {
 	FILE *fp = fopen(fill_by, "r");
-	if (fp != 0) {
+	if (fp != NULL) {
 	    bool filled = FALSE;
 	    move(1, 0);
 	    while ((ch = fgetc(fp)) != EOF) {
@@ -570,7 +577,7 @@ main(int argc, char *argv[])
 	int x = 0;
 	int color = 0;
 	int altchars = 0;
-	bool dirty = use_colors || (fill_by != 0);
+	bool dirty = use_colors || (fill_by != NULL);
 
 	while (!done) {
 	    switch (get_command(color, which, last)) {
@@ -648,7 +655,7 @@ main(int argc, char *argv[])
 
 #else
 int
-main(int argc GCC_UNUSED, char *argv[]GCC_UNUSED)
+main(void)
 {
     printf("This program requires the screen-dump functions\n");
     ExitProgram(EXIT_FAILURE);
