@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2018-2022,2023 Thomas E. Dickey                                *
+ * Copyright 2018-2024,2025 Thomas E. Dickey                                *
  * Copyright 1998-2017,2018 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -30,7 +30,7 @@
 /****************************************************************************
  *  Author: Thomas E. Dickey                    1996-on                     *
  ****************************************************************************/
-/* $Id: test.priv.h,v 1.217 2023/11/11 00:23:26 tom Exp $ */
+/* $Id: test.priv.h,v 1.221 2025/01/18 15:07:25 tom Exp $ */
 
 #ifndef __TEST_PRIV_H
 #define __TEST_PRIV_H 1
@@ -752,25 +752,27 @@ extern int optind;
 ," -V       show version of curses"
 
 #if HAVE_CURSES_VERSION
-#define format_version(buffer) strcpy(buffer, curses_version())
+#define format_version(buffer, size) strcpy(buffer, curses_version())
 #elif defined(NCURSES_VERSION_MAJOR) && defined(NCURSES_VERSION_MINOR) && defined(NCURSES_VERSION_PATCH)
-#define format_version(buffer) sprintf(buffer, "ncurses %d.%d.%d", \
-	NCURSES_VERSION_MAJOR, \
-	NCURSES_VERSION_MINOR, \
-	NCURSES_VERSION_PATCH)
+#define format_version(buffer, size) \
+	_nc_SPRINTF(buffer, _nc_SLIMIT(size) "ncurses %d.%d.%d", \
+		    NCURSES_VERSION_MAJOR, \
+		    NCURSES_VERSION_MINOR, \
+		    NCURSES_VERSION_PATCH)
 #else
-#define format_version(buffer) strcpy(buffer, "ncurses-examples")
+#define format_version(buffer, size) strcpy(buffer, "ncurses-examples")
 #endif
 
 #define VERSION_COMMON() \
 static char *version_common(char **argv) { \
-	char *base = argv[0]; \
-	char *part = strrchr(base, '/'); \
-	char *result = malloc(strlen(base) + 80); \
+	const char *base = argv[0]; \
+	const char *part = strrchr(base, '/'); \
+	size_t need = strlen(base) + 80; \
+	char *result = malloc(need); \
 	if (result != NULL) { \
 	    if (part++ == NULL) part = base; \
-	    sprintf(result, "%.20s: ", part); \
-	    format_version(result + strlen(result)); \
+	    _nc_SPRINTF(result, _nc_SLIMIT(need) "%.20s: ", part); \
+	    format_version(result + strlen(result), need - strlen(result)); \
 	} \
 	return result; \
 } \
@@ -850,6 +852,12 @@ extern "C" {
 #define _nc_SLIMIT(n)		/* nothing */
 #endif
 
+/*
+ * To make them easier to find, user-defined capabilities used within ncurses
+ * should be tagged with this macro:
+ */
+#define UserCap(name) #name
+
 /*
  * X/Open Curses does not define the arrays of terminfo/termcap names as SVr4
  * curses did, and some implementations provide them anyway, but undeclared.
@@ -914,8 +922,8 @@ extern int TABSIZE;
  * Workaround in case getcchar() returns a positive value when the source
  * string produces only a L'\0'.
  */
-#define TEST_CCHAR(s, count, then_stmt, else_stmt) \
-	if ((count = getcchar(s, NULL, NULL, NULL, NULL)) > 0) { \
+#define TEST_CCHAR(s, then_stmt, else_stmt) \
+	if (getcchar(s, NULL, NULL, NULL, NULL) > 0) { \
 	    wchar_t test_wch[CCHARW_MAX + 2]; \
 	    attr_t test_attrs; \
 	    NCURSES_PAIRS_T test_pair; \
@@ -1045,12 +1053,12 @@ extern int TABSIZE;
 #define EXIT_FAILURE 1
 #endif
 
-#undef _NC_WINDOWS
+#undef _NC_WINDOWS_NATIVE
 #if (defined(_WIN32) || defined(_WIN64))
-#define _NC_WINDOWS 1
+#define _NC_WINDOWS_NATIVE 1
 #endif
 
-#if defined(_NC_WINDOWS) || defined(USE_WIN32CON_DRIVER)
+#if defined(_NC_WINDOWS_NATIVE) || defined(USE_WIN32CON_DRIVER)
 
 #if defined(PDCURSES)
 #ifdef WINVER
@@ -1112,14 +1120,14 @@ extern int TABSIZE;
 #if HAVE_CLOCK_GETTIME
 # define GetClockTime(t) clock_gettime(CLOCK_REALTIME, t)
 # define TimeType struct timespec
-# define TimeScale 1000000000L		/* 1e9 */
+# define TimeScale 1000000000L	/* 1e9 */
 # define ElapsedSeconds(b,e) \
 	    (double) (((e)->tv_sec - (b)->tv_sec) \
 		    + ((e)->tv_nsec - (b)->tv_nsec) / TimeScale)
 #elif HAVE_GETTIMEOFDAY
 # define GetClockTime(t) gettimeofday(t, 0)
 # define TimeType struct timeval
-# define TimeScale 1000000L		/* 1e6 */
+# define TimeScale 1000000L	/* 1e6 */
 # define ElapsedSeconds(b,e) \
 	    (double) (((e)->tv_sec - (b)->tv_sec) \
 		    + ((e)->tv_usec - (b)->tv_usec) / TimeScale)
@@ -1182,7 +1190,7 @@ extern char *_nc_strstr(const char *, const char *);
 #define InitAndCatch(init,handler) do { init; CATCHALL(handler); } while (0)
 #endif
 
-#if defined(_NC_WINDOWS) || defined(USE_WIN32CON_DRIVER)
+#if defined(_NC_WINDOWS_NATIVE) || defined(USE_WIN32CON_DRIVER)
 #define SetupAlarm(opt)	(void)opt
 #else
 #define SetupAlarm(opt)	if (opt) alarm((unsigned)opt)
